@@ -4,6 +4,7 @@ from anndata import AnnData
 from ._datastructures import AirrCell
 from typing import (
     Iterable,
+    List,
     Mapping,
     MutableMapping,
     Sequence,
@@ -21,7 +22,12 @@ from pathlib import Path
 import airr
 from ..util import _doc_params, _is_true, _is_true2, _translate_dna_to_protein, _is_na2
 from ._convert_anndata import from_airr_cells, to_airr_cells, _sanitize_anndata
-from ._util import doc_working_model, _IOLogger, _check_upgrade_schema
+from ._util import (
+    doc_working_model,
+    _IOLogger,
+    _check_upgrade_schema,
+    _read_airr_rearrangement_df,
+)
 from .._compat import Literal
 from airr import RearrangementSchema
 import itertools
@@ -54,17 +60,17 @@ def _cdr3_from_junction(junction_aa, junction_nt):
     Should the conserved residues not equal to C and W/F, then the chain
     is non-productive and we set CDR3 to None.
 
-    See also https://github.com/icbi-lab/scirpy/pull/290.
+    See also https://github.com/scverse/scirpy/pull/290.
     """
     cdr3_aa, cdr3_nt = None, None
     if (
-        junction_aa is not None
+        not _is_na2(junction_aa)
         and junction_aa[0] == "C"
         and junction_aa[-1] in ("W", "F")
     ):
         cdr3_aa = junction_aa[1:-1]
     if (
-        junction_nt is not None
+        not _is_na2(junction_nt)
         and _translate_dna_to_protein(junction_nt[:3]) == "C"
         and _translate_dna_to_protein(junction_nt[-3:]) in ("W", "F")
     ):
@@ -270,7 +276,7 @@ def read_10x_vdj(
         Per default, this includes a subset of columns relevant for a typical
         scirpy analysis, to keep `adata.obs` a bit cleaner. Defaults to {include_fields}.
         Set this to `None` to include all columns.
-    
+
 
     Returns
     -------
@@ -404,7 +410,9 @@ def read_tracer(path: Union[str, Path]) -> AnnData:
     include_fields=f"""`({",".join([f'"{x}"' for x in DEFAULT_AIRR_FIELDS])})`""",
 )
 def read_airr(
-    path: Union[str, Sequence[str], Path, Sequence[Path]],
+    path: Union[
+        str, Sequence[str], Path, Sequence[Path], pd.DataFrame, Sequence[pd.DataFrame]
+    ],
     use_umi_count_col: Union[bool, Literal["auto"]] = "auto",
     infer_locus: bool = True,
     cell_attributes: Collection[str] = DEFAULT_AIRR_CELL_ATTRIBUTES,
@@ -433,6 +441,7 @@ def read_airr(
         Path to the AIRR rearrangement tsv file. If different
         chains are split up into multiple files, these can be specified
         as a List, e.g. `["path/to/tcr_alpha.tsv", "path/to/tcr_beta.tsv"]`.
+        Alternatively, this can be a pandas data frame.
     use_umi_count_col
         Whether to add UMI counts from the non-strandard (but common) `umi_count`
         column. When this column is used, the UMI counts are moved over to the
@@ -460,7 +469,7 @@ def read_airr(
     logger = _IOLogger()
 
     if isinstance(path, (str, Path, pd.DataFrame)):
-        path: list = [path]
+        path: List[Union[str, Path, pd.DataFrame]] = [path]  # type: ignore
 
     def _decide_use_umi_count_col(chain_dict):
         """Logic to decide whether or not to use counts form the `umi_counts` column."""
@@ -478,11 +487,11 @@ def read_airr(
         else:
             return False
 
-    for tmp_path in path:
-        if isinstance(tmp_path, pd.DataFrame):
-            iterator = tmp_path.to_dict(orient="records")
+    for tmp_path_or_df in path:
+        if isinstance(tmp_path_or_df, pd.DataFrame):
+            iterator = _read_airr_rearrangement_df(tmp_path_or_df)
         else:
-            iterator = airr.read_rearrangement(str(tmp_path))
+            iterator = airr.read_rearrangement(str(tmp_path_or_df))
 
         for chain_dict in iterator:
             cell_id = chain_dict.pop("cell_id")
@@ -514,7 +523,7 @@ def read_airr(
 
 
 def _infer_locus_from_gene_names(chain_dict):
-    """Infer the IGMT locus name from VDJ calls"""
+    """Infer the IMGT locus name from VDJ calls"""
     keys = ["v_call", "d_call", "j_call", "c_call"]
 
     # TRAV.*/DV is misleading as it actually points to a delta locus
@@ -780,3 +789,79 @@ def from_dandelion(dandelion, transfer: bool = False, **kwargs) -> AnnData:
             adata, dandelion
         )  # need to make a version that is not so verbose?
     return adata
+
+
+@_doc_params(doc_working_model=doc_working_model)
+def read_bd_rhapsody(path: Union[str, Path], dominant=False) -> AnnData:
+    """\
+    Read :term:`IR` data from the BD Rhapsody Analysis Pipeline.
+
+    Supports `*_perCellChain.csv`, `*_perCellChain_unfiltered.csv`, `*_VDJ_Dominant_Contigs.csv`, and
+    `*_VDJ_Unfiltered_Contigs.csv` files. The applicable filename depends your version of the BD Rhapsody pipeline.
+
+    .. note::
+
+        More recent versions of the pipeline generate data in standardized `AIRR Rearragement format <https://docs.airr-community.org/en/latest/datarep/rearrangements.html>`_.
+        If you have a chance to do so, we recommend reanalysing your data with the most recent version of the
+        BD Rhapsody pipeline and read output filese with :func:`scirpy.io.read_airr`.
+
+    `*_perCell` files are currently not supported, follow the :ref:`IO Tutorial <importing-custom-formats>` to import
+    custom formats and make use of `this snippet <https://github.com/scverse/scirpy/blob/8de293fd54125a00f8ff6fd5f8a4cb232d7a51e6/scirpy/io/_io_bdrhapsody.py#L8-L44>`_
+
+    {doc_working_model}
+
+    Parameters
+    ----------
+    path:
+        Path to the `perCellChain` or `Contigs` file generated by the BD Rhapsody analysis pipeline. May be gzipped.
+
+    Returns
+    -------
+    A :class:`~anndata.AnnData` instance with AIRR information stored in `obs`.
+    """
+    df = pd.read_csv(path, comment="#", index_col=0)
+
+    def _translate_locus(locus):
+        return {
+            "TCR_Alpha": "TRA",
+            "TCR_Beta": "TRB",
+            "TCR_Gamma": "TRG",
+            "TCR_Delta": "TRD",
+            "BCR_Lambda": "IGL",
+            "BCR_Kappa": "IGK",
+            "BCR_Heavy": "IGH",
+        }[locus]
+
+    def _get(row, field):
+        try:
+            return row[field]
+        except KeyError:
+            # in "Dominant_Contigs", the fields end with "_Dominant"
+            return row[f"{field}_Dominant"]
+
+    airr_cells = {}
+    for idx, row in df.iterrows():
+        idx = str(idx)
+        if idx not in airr_cells:
+            airr_cells[idx] = AirrCell(cell_id=idx)
+        tmp_cell = airr_cells[idx]
+        tmp_chain = AirrCell.empty_chain_dict()
+        tmp_chain.update(
+            {
+                "locus": _translate_locus(row["Chain_Type"]),
+                "v_call": _get(row, "V_gene"),
+                "d_call": _get(row, "D_gene"),
+                "j_call": _get(row, "J_gene"),
+                "c_call": _get(row, "C_gene"),
+                # strictly, this would have to go the the "cdr3" field as the conserved residues
+                # are not contained. However we only work with `junction` in scirpy
+                "junction": _get(row, "CDR3_Nucleotide"),
+                "junction_aa": _get(row, "CDR3_Translation"),
+                "productive": row["Productive"],
+                "consensus_count": row["Read_Count"],
+                "duplicate_count": row["Molecule_Count"],
+            }
+        )
+        tmp_cell.add_chain(tmp_chain)
+
+    return from_airr_cells(airr_cells.values())
